import { Grid } from "@mui/material";
import { useParams } from "react-router-dom";
import { useSelector, useDispatch } from "react-redux";
import { useCallback, useEffect, useMemo, useState } from "react";
import { createMember, deleteMember, fetchProject } from "../../store/actions/projectsActions";
import { fetchUsers, fetchUsersAllFields } from "../../store/actions/usersActions";
import ProjectUsersColumnsWrapper from "../../components/ProjectComponents/ProjectUsersColumnsWrapper/ProjectUsersColumnsWrapper";
import ProjectInfo from "../../components/ProjectComponents/ProjectInfo/ProjectInfo";
import DefaultModal from "../../components/UI/DefaultModal/DefaultModal"
import NewMemberModalContent from "../../components/ProjectComponents/NewMemberModalContent/NewMemberModalContent";


const FullProject = () => {

    const { project } = useSelector(state => state.projects);
    const [modal, setModal] = useState(false)
    const [newMember, setNewMember] = useState(null)
    const { user, users } = useSelector(state => state.users);

    const dispatch = useDispatch();
    const params = useParams()

    useEffect(() => {
        dispatch(fetchProject(params.id))
        dispatch(fetchUsers())
    }, [params.id, dispatch]);

    const members = useMemo(() => {
        return project?.project?.members || []
    }, [project])

    const onChangeRoleHandler = useCallback((e, value) => {
        setNewMember((prevState) => { return { ...prevState, roleProject: value?.value } });
    }, []);

    const onChangeNewMemberHandler = useCallback((e, value) => {
        setNewMember((prevState) => { return { ...prevState, userId: value?.id } });
    }, []);

    const handleOpen = useCallback(async () => {
        setModal(true)
    }, [])

    const handleClose = useCallback(() => {
        setModal(false)
        setNewMember(null)
    }, [])

    const createNewMemberHandler = useCallback(()=>{
        dispatch(createMember({...newMember, projectId: project?.project?.id}))
        setModal(false)
    }, [dispatch, newMember, project?.project?.id])

    const deleteMemberHandler = useCallback((id)=>{
        dispatch(deleteMember(id, project?.project?.id))
    }, [dispatch, project?.project?.id])

    return <>
        <Grid>

            <ProjectInfo
                project={project.project}
                handleOpen={handleOpen}
            />

            <ProjectUsersColumnsWrapper
                members={members}
                deleteMemberHandler={deleteMemberHandler}
            />

            <DefaultModal
                modal={modal}
                handleClose={() => { handleClose() }}
            >
                <NewMemberModalContent
                    users={users}
                    onChangeNewMemberHandler={onChangeNewMemberHandler}
                    onChangeRoleHandler={onChangeRoleHandler}
                    newMember={newMember}
                    handleClose={handleClose}
                    createNewMemberHandler={createNewMemberHandler}
                >

                </NewMemberModalContent>
            </DefaultModal>

        </Grid>
    </>
};

export default FullProject;
